/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestDbChecker.java 6218 2010-03-29 08:25:18Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.tests.dbchecker;

import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.ow2.jasmine.event.beans.JasmineEventNotification;
import org.ow2.jasmine.event.beans.JasmineEventNotificationDetail;
import org.ow2.jasmine.event.beans.JasmineEventNotificationExecutedAction;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventNotificationSLBRemote;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

/**
 * Test class that is used to check if some notifications are present in the
 * database. It will use the EJB3 that is making persistent the data
 *
 * @author Florent Benoit
 */
public class TestDbChecker {

    private JasmineEventNotificationSLBRemote eventNotificationBean = null;

    @BeforeClass
    public void getBean() throws Exception {
        long stopTime = System.currentTimeMillis() + 60 * 1000;
        while (System.currentTimeMillis() < stopTime) {
            try {
                this.eventNotificationBean = (JasmineEventNotificationSLBRemote) getContext().lookup("db-ejb/notification");
                return;
            } catch (NamingException e) {
                Thread.sleep(1000);
            }
        }

        Assert.fail("cannot find db-ejb/notification (" + this.eventNotificationBean + ")");
    }

    @Test
    public void checkNotifications() throws Exception {
        List<JasmineEventNotification> notifications = null;

        long stopTime = System.currentTimeMillis() + 60 * 1000;
        while ((notifications == null || notifications.size() != 2) && System.currentTimeMillis() < stopTime) {
            notifications = eventNotificationBean.getAllNotifications();
        }

        Assert.assertNotNull(notifications, "notifications is null");
        Assert.assertEquals(notifications.size(), 2, "we should have two notifications");

        // Now, check that the 2 notifications are valid
        JasmineEventNotification notification1 = notifications.get(0);
        JasmineEventNotification notification2 = notifications.get(1);

        Assert.assertFalse(notification1.getStatus().booleanValue());
        Assert.assertFalse(notification2.getStatus().booleanValue());

        if ("Info".equals(notification1.getLevel())) {
            // check other notification
            Assert.assertEquals(notification2.getLevel(), "Error");
        } else if ("Error".equals(notification1.getLevel())) {
            Assert.assertEquals(notification2.getLevel(), "Info");
        } else {
            Assert.fail("Invalid notification1 level: " + notification1.getLevel());
        }
    }

    /**
     * Check that the date is correct
     */
    @Test(dependsOnMethods={"checkNotifications"})
    public void testNotificationContents() {
        List<JasmineEventNotification> notifications = eventNotificationBean.getAllNotifications();

        JasmineEventNotification notif0 = null;
        JasmineEventNotification notif1 = null;

        if ("Info".equals(notifications.get(0).getLevel())) {
            notif0 = notifications.get(0);
            notif1 = notifications.get(1);
        } else {
            notif0 = notifications.get(1);
            notif1 = notifications.get(0);
        }

        Assert.assertEquals(notif0.getRule(), "test-rule");
        Assert.assertEquals(notif1.getRule(), "test-rule");

        // details of notification 0
        // 4 details
        List<JasmineEventNotificationDetail> details = notif0.getDetails();
        Assert.assertEquals(details.size(), 4);
        JasmineEventNotificationDetail domainDetail = details.get(0);
        JasmineEventNotificationDetail serverDetail = details.get(1);
        JasmineEventNotificationDetail timestampDetail = details.get(2);
        JasmineEventNotificationDetail valueDetail = details.get(3);
        Assert.assertEquals(domainDetail.getLabel(), "domain");
        Assert.assertEquals(domainDetail.getMessage(), "myDomain");

        Assert.assertEquals(serverDetail.getLabel(), "server");
        Assert.assertEquals(serverDetail.getMessage(), "florentServer");

        Assert.assertEquals(timestampDetail.getLabel(), "timestamp");
        Assert.assertTrue(timestampDetail.getMessage().contains("Wed Sep 09"));
        Assert.assertTrue(timestampDetail.getMessage().contains("2009"));

        Assert.assertEquals(valueDetail.getLabel(), "value");
        Assert.assertEquals(valueDetail.getMessage(), "myValue2");


        // details of notification 1
        // 4 details
        details = notif1.getDetails();
        Assert.assertEquals(4, details.size());
        domainDetail = details.get(0);
        serverDetail = details.get(1);
        timestampDetail = details.get(2);
        valueDetail = details.get(3);
        Assert.assertEquals(domainDetail.getLabel(), "domain");
        Assert.assertEquals(domainDetail.getMessage(), "myDomain");

        Assert.assertEquals(serverDetail.getLabel(), "server");
        Assert.assertEquals(serverDetail.getMessage(), "florentServer");

        Assert.assertEquals(timestampDetail.getLabel(), "timestamp");
        Assert.assertTrue(timestampDetail.getMessage().contains("Wed Sep 09"));
        Assert.assertTrue(timestampDetail.getMessage().contains("2009"));

        Assert.assertEquals(valueDetail.getLabel(), "value");
        Assert.assertEquals(valueDetail.getMessage(), "myValueSentByDrools");
    }

    /**
     * Check that the filtering with null works
     */
    @Test(dependsOnMethods={"checkNotifications"})
    public void testGetEventNotifications() {
        List<JasmineEventNotification> notifications;

        // test getter without any notification
        notifications = eventNotificationBean.getEventNotifications(2, null, "*");
        Assert.assertEquals(notifications.size(), 2);
    }

    /**
     * Check that acknowledgement works and that filtering works
     */
    @Test(dependsOnMethods={"testGetEventNotifications"})
    public void testAcknowledgeNotification() {
        List<JasmineEventNotification> notifications;

        // testAction hasn't been executed on any notification yet,
        // this call should return all notifications
        notifications = eventNotificationBean.getEventNotifications(2, "testAction", "*");
        Assert.assertEquals(notifications.size(), 2);

        JasmineEventNotification notificationOneHourAgo = notifications.get(0);
        JasmineEventNotificationExecutedAction actionOneHourAgo = new JasmineEventNotificationExecutedAction("testComment",
            "testAction", new Date(System.currentTimeMillis() - 1 * 60 * 60 * 1000), "testUser");
        eventNotificationBean.saveExecutedAction(notificationOneHourAgo.getId(), actionOneHourAgo);
        eventNotificationBean.updateNotificationStatus(notificationOneHourAgo.getId(), true);

        JasmineEventNotification notificationThreeHoursAgo = notifications.get(1);
        JasmineEventNotificationExecutedAction actionThreeHoursAgo = new JasmineEventNotificationExecutedAction("testComment",
            "testAction", new Date(System.currentTimeMillis() - 3 * 60 * 60 * 1000), "testUser");
        eventNotificationBean.saveExecutedAction(notificationThreeHoursAgo.getId(), actionThreeHoursAgo);
        eventNotificationBean.updateNotificationStatus(notificationThreeHoursAgo.getId(), true);

        // this returns all notifications sent less than two hours ago
        //
        // since we've just sent the notifications, it returns them all
        // AND they're all acknowledged
        notifications = eventNotificationBean.getEventNotifications(2, null, "*");
        Assert.assertEquals(notifications.size(), 2);
        int acknowledgedNotifications = 0;
        for (JasmineEventNotification notification : notifications) {
            if (notification.getStatus()) {
                acknowledgedNotifications++;
            }
        }
        Assert.assertEquals(acknowledgedNotifications, 2, notifications.toString());

        // this returns all notifications sent less than -1 hours ago
        //
        // result is empty, notifications can't have been sent in the future
        notifications = eventNotificationBean.getEventNotifications(-1, null, "*");
        Assert.assertEquals(notifications.size(), 0);

        // the acknowledgment action called testAction has been executed on
        // two notifications:
        //      one 1 hour ago
        //      another 3 hours ago
        //
        // this call should return the notification acknowledged 1 hour ago
        // and ignore the one acknowledged 3 hours ago
        notifications = eventNotificationBean.getEventNotifications(2, "testAction", "*");
        Assert.assertEquals(notifications.size(), 1);
        Assert.assertEquals(notifications.get(0).getId(), notificationOneHourAgo.getId(),
            "return notification id is not notificationOneHourAgo (" + notificationOneHourAgo + ")");
    }

    protected Context getContext() throws NamingException {
        Hashtable<String, Object> env = new Hashtable<String, Object>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, org.ow2.easybeans.component.smartclient.spi.SmartContextFactory.class.getName());
        env.put(Context.PROVIDER_URL, "smart://localhost:" + System.getProperty("smartFactory.port", "2503"));
        return new InitialContext(env);
    }
}
